@@ -22,6 +22,8 @@ our @EXPORT_OK = qw(
 	import_sv
 	sass_compile
 	sass_compile_file
+	libsass_version
+	sass2scss_version
 	SASS_COMMA
 	SASS_SPACE
 	SASS_ERROR
@@ -46,7 +48,7 @@ our @EXPORT = qw(
 	SASS2SCSS_CONVERT_COMMENT
 );
 
-our $VERSION = "v3.0.2";
+our $VERSION = "v3.1.0";
 
 require XSLoader;
 XSLoader::load('CSS::Sass', $VERSION);
@@ -83,7 +85,7 @@ sub sass_compile {
                                                                 : (include_paths => join($^O eq 'MSWin32' ? ';' : ':',
                                                                                          @{$options{include_paths}})),
                                      });
-    wantarray ? ($r->{output_string}, $r->{error_message}, $r->{source_map_string}) : $r->{output_string}
+    wantarray ? ($r->{output_string}, $r->{error_message}, $r) : $r->{output_string}
 }
 
 sub sass_compile_file {
@@ -99,23 +101,23 @@ sub sass_compile_file {
                                                                      : (include_paths => join($^O eq 'MSWin32' ? ';' : ':',
                                                                                               @{$options{include_paths}})),
                                           });
-    wantarray ? ($r->{output_string}, $r->{error_message}, $r->{source_map_string}) : $r->{output_string}
+    wantarray ? ($r->{output_string}, $r->{error_message}, $r) : $r->{output_string}
 }
 
 sub compile {
     my ($self, $sass_code) = @_;
-    my ($compiled, $srcmap);
-    ($compiled, $self->{last_error}, $srcmap) = sass_compile($sass_code, %{$self->options});
+    my ($compiled, $stats);
+    ($compiled, $self->{last_error}, $stats) = sass_compile($sass_code, %{$self->options});
     croak $self->{last_error} if $self->{last_error} && !$self->options->{dont_die};
-    wantarray ? ($compiled, $srcmap) : $compiled
+    wantarray ? ($compiled, $stats) : $compiled
 }
 
 sub compile_file {
     my ($self, $sass_file) = @_;
-    my ($compiled, $srcmap);
-    ($compiled, $self->{last_error}, $srcmap) = sass_compile_file($sass_file, %{$self->options});
+    my ($compiled, $stats);
+    ($compiled, $self->{last_error}, $stats) = sass_compile_file($sass_file, %{$self->options});
     croak $self->{last_error} if $self->{last_error} && !$self->options->{dont_die};
-    wantarray ? ($compiled, $srcmap) : $compiled
+    wantarray ? ($compiled, $stats) : $compiled
 }
 
 1;
@@ -280,7 +282,7 @@ line the code corresponds to.
 
 Setting this option enables the source-map generating. The file will not
 actually be created, but its content will be returned to the caller. It
-will also enable sourceMappingURL comment by default. See C<omit_src_map_url>.
+will also enable sourceMappingURL comment by default. See C<no_src_map_url>.
 
 =item C<source_map_embed>
 
@@ -293,7 +295,7 @@ Embeds the content of each source inside a C<sourcesContent> property in the
 source-map json. Setting this option along with C<source_map_embed> allows
 for a completely self-contained source-map.
 
-=item C<omit_src_map_url>
+=item C<no_src_map_url>
 
 Set to C<true> to omit the sourceMappingURL comment from the output css.
 Setting this options makes C<source_map_embed> useless.
@@ -352,6 +354,27 @@ Then the ouput would be:
 
     some_rule: Well, hello;
 
+=item Custom C<importer>
+
+This is a function implemented in Perl that gets called for every @import statement. This
+feature is in an experimental stage and you have to be careful to return the expected
+structure. You can return multiple imports from one call to make it possible to
+implement globbing importers etc. If you omit $data, libsass will try to load the
+given path itself. It will go through the normal lockup algorithm as it would had
+encountered the "virtual" import statement on its own. $scope holds the current
+import path. Imports in css are meant to be relative to the parent scope, so you
+can use it to create absolute urls or paths within the context your working with.
+
+A simple example:
+
+    importer => sub {
+      my ($import, $scope) = @_;
+      return [
+        # [ $real_path ] or [ $virtual_path, $data ],
+        [ "http://xyz/file", "div { color: red; }" ],
+      ];
+    }
+
 =item C<Sass_Value> Types
 
 Sass knowns various C<Sass_Value> types. We export the constants for completeness.
