@@ -1,16 +1,10 @@
 #ifdef _WIN32
-#include <direct.h>
-#define getcwd _getcwd
 #define PATH_SEP ';'
 #else
-#include <unistd.h>
 #define PATH_SEP ':'
 #endif
 
-#ifndef SASS_AST
 #include "ast.hpp"
-#endif
-
 #include "context.hpp"
 #include "constants.hpp"
 #include "parser.hpp"
@@ -21,6 +15,7 @@
 #include "expand.hpp"
 #include "eval.hpp"
 #include "contextualize.hpp"
+#include "cssize.hpp"
 #include "extend.hpp"
 #include "remove_placeholders.hpp"
 #include "copy_c_str.hpp"
@@ -28,10 +23,7 @@
 #include "functions.hpp"
 #include "backtrace.hpp"
 #include "sass2scss.h"
-
-#ifndef SASS_PRELEXER
 #include "prelexer.hpp"
-#endif
 
 #include <iomanip>
 #include <iostream>
@@ -44,16 +36,27 @@ namespace Sass {
   using std::cerr;
   using std::endl;
 
+  Sass_Queued::Sass_Queued(const string& load_path, const string& abs_path, const char* source)
+  {
+    this->load_path = load_path;
+    this->abs_path = abs_path;
+    this->source = source;
+  }
+
+
   Context::Context(Context::Data initializers)
   : mem(Memory_Manager<AST_Node>()),
     source_c_str            (initializers.source_c_str()),
     sources                 (vector<const char*>()),
     include_paths           (initializers.include_paths()),
-    queue                   (vector<pair<string, const char*> >()),
+    queue                   (vector<Sass_Queued>()),
     style_sheets            (map<string, Block*>()),
     source_map              (resolve_relative_path(initializers.output_path(), initializers.source_map_file(), get_cwd())),
-    c_functions             (vector<Sass_C_Function_Descriptor>()),
+    c_functions             (vector<Sass_C_Function_Callback>()),
+    indent                  (initializers.indent()),
+    linefeed                (initializers.linefeed()),
     image_path              (initializers.image_path()),
+    input_path              (make_canonical_path(initializers.input_path())),
     output_path             (make_canonical_path(initializers.output_path())),
     source_comments         (initializers.source_comments()),
     output_style            (initializers.output_style()),
@@ -62,6 +65,7 @@ namespace Sass {
     source_map_contents     (initializers.source_map_contents()),
     omit_source_map_url     (initializers.omit_source_map_url()),
     is_indented_syntax_src  (initializers.is_indented_syntax_src()),
+    importer                (initializers.importer()),
     names_to_colors         (map<string, Color*>()),
     colors_to_names         (map<int, string>()),
     precision               (initializers.precision()),
@@ -70,6 +74,12 @@ namespace Sass {
   {
     cwd = get_cwd();
 
+    // enforce some safe defaults
+    // used to create relative file links
+    if (input_path == "") input_path = "stdin";
+    if (output_path == "") output_path = "stdout";
+
+    include_paths.push_back(cwd);
     collect_include_paths(initializers.include_paths_c_str());
     collect_include_paths(initializers.include_paths_array());
 
@@ -88,6 +98,8 @@ namespace Sass {
   {
     // everything that gets put into sources will be freed by us
     for (size_t i = 0; i < sources.size(); ++i) delete[] sources[i];
+    for (size_t n = 0; n < import_stack.size(); ++n) sass_delete_import(import_stack[n]);
+    sources.clear(); import_stack.clear();
   }
 
   void Context::setup_color_map()
@@ -95,22 +107,25 @@ namespace Sass {
     size_t i = 0;
     while (color_names[i]) {
       string name(color_names[i]);
-      Color* value = new (mem) Color("[COLOR TABLE]", Position(),
-                                     color_values[i*3],
-                                     color_values[i*3+1],
-                                     color_values[i*3+2]);
+      Color* value = new (mem) Color(ParserState("[COLOR TABLE]"),
+                                     color_values[i*4],
+                                     color_values[i*4+1],
+                                     color_values[i*4+2],
+                                     color_values[i*4+3]);
       names_to_colors[name] = value;
-      int numval = color_values[i*3]*0x10000;
-      numval += color_values[i*3+1]*0x100;
-      numval += color_values[i*3+2];
-      colors_to_names[numval] = name;
+      // only map fully opaque colors
+      if (color_values[i*4+3] >= 1) {
+        int numval = color_values[i*4]*0x10000;
+        numval += color_values[i*4+1]*0x100;
+        numval += color_values[i*4+2];
+        colors_to_names[numval] = name;
+      }
       ++i;
     }
   }
 
   void Context::collect_include_paths(const char* paths_str)
   {
-    include_paths.push_back(cwd);
 
     if (paths_str) {
       const char* beg = paths_str;
@@ -134,27 +149,21 @@ namespace Sass {
     }
   }
 
-  void Context::collect_include_paths(const char* paths_array[])
+  void Context::collect_include_paths(const char** paths_array)
   {
-    include_paths.push_back(get_cwd());
     if (*include_paths.back().rbegin() != '/') include_paths.back() += '/';
-
-    // if (paths_array) {
-    //   for (size_t i = 0; paths_array[i]; ++i) {
-    //     string path(paths_array[i]);
-    //     if (!path.empty()) {
-    //       if (*path.rbegin() != '/') path += '/';
-    //       include_paths.push_back(path);
-    //     }
-    //   }
-    // }
+    if (paths_array) {
+      for (size_t i = 0; paths_array[i]; i++) {
+        collect_include_paths(paths_array[i]);
+      }
+    }
   }
 
-  void Context::add_source(const string& load_path, const string& abs_path, const char* contents)
+  void Context::add_source(string load_path, string abs_path, const char* contents)
   {
     sources.push_back(contents);
     included_files.push_back(abs_path);
-    queue.push_back(make_pair(load_path, contents));
+    queue.push_back(Sass_Queued(load_path, abs_path, contents));
     source_map.source_index.push_back(sources.size() - 1);
     include_links.push_back(resolve_relative_path(abs_path, source_map_file, cwd));
   }
@@ -209,40 +218,13 @@ namespace Sass {
   void register_function(Context&, Signature sig, Native_Function f, size_t arity, Env* env);
   void register_overload_stub(Context&, string name, Env* env);
   void register_built_in_functions(Context&, Env* env);
-  void register_c_functions(Context&, Env* env, Sass_C_Function_Descriptor*);
-  void register_c_function(Context&, Env* env, Sass_C_Function_Descriptor);
+  void register_c_functions(Context&, Env* env, Sass_C_Function_List);
+  void register_c_function(Context&, Env* env, Sass_C_Function_Callback);
 
-  char* Context::compile_file()
+  char* Context::compile_block(Block* root)
   {
-    Block* root = 0;
-    for (size_t i = 0; i < queue.size(); ++i) {
-      Parser p(Parser::from_c_str(queue[i].second, *this, queue[i].first, Position(1 + i, 1, 1)));
-      Block* ast = p.parse();
-      if (i == 0) root = ast;
-      style_sheets[queue[i].first] = ast;
-    }
-    Env tge;
-    Backtrace backtrace(0, "", Position(), "");
-    register_built_in_functions(*this, &tge);
-    for (size_t i = 0, S = c_functions.size(); i < S; ++i) {
-    	register_c_function(*this, &tge, c_functions[i]);
-    }
-    Eval eval(*this, &tge, &backtrace);
-    Contextualize contextualize(*this, &eval, &tge, &backtrace);
-    Expand expand(*this, &eval, &contextualize, &tge, &backtrace);
-    // Inspect inspect(this);
-    // Output_Nested output_nested(*this);
-
-    root = root->perform(&expand)->block();
-    if (!subset_map.empty()) {
-      Extend extend(*this, subset_map);
-      root->perform(&extend);
-    }
-
-    Remove_Placeholders remove_placeholders(*this);
-    root->perform(&remove_placeholders);
-
     char* result = 0;
+    if (!root) return 0;
     switch (output_style) {
       case COMPRESSED: {
         Output_Compressed output_compressed(this);
@@ -259,16 +241,84 @@ namespace Sass {
         root->perform(&output_nested);
         string output = output_nested.get_buffer();
         if (source_map_file != "" && !omit_source_map_url) {
-          output += "\n" + format_source_mapping_url(source_map_file);
+          output += linefeed + format_source_mapping_url(source_map_file);
         }
         result = copy_c_str(output.c_str());
 
       } break;
     }
-
     return result;
   }
 
+  Block* Context::parse_file()
+  {
+    Block* root = 0;
+    for (size_t i = 0; i < queue.size(); ++i) {
+      struct Sass_Import* import = sass_make_import(
+        queue[i].load_path.c_str(),
+        queue[i].abs_path.c_str(),
+        0, 0
+      );
+      import_stack.push_back(import);
+      Parser p(Parser::from_c_str(queue[i].source, *this, ParserState(queue[i].abs_path, i)));
+      Block* ast = p.parse();
+      sass_delete_import(import_stack.back());
+      import_stack.pop_back();
+      if (i == 0) root = ast;
+      style_sheets[queue[i].load_path] = ast;
+    }
+    if (root == 0) return 0;
+    Env tge;
+    Backtrace backtrace(0, ParserState("", 0), "");
+    register_built_in_functions(*this, &tge);
+    for (size_t i = 0, S = c_functions.size(); i < S; ++i) {
+      register_c_function(*this, &tge, c_functions[i]);
+    }
+    Eval eval(*this, &tge, &backtrace);
+    Contextualize contextualize(*this, &eval, &tge, &backtrace);
+    Expand expand(*this, &eval, &contextualize, &tge, &backtrace);
+    Cssize cssize(*this, &tge);
+    // Inspect inspect(this);
+    // Output_Nested output_nested(*this);
+
+    root = root->perform(&expand)->block();
+    root = root->perform(&cssize)->block();
+    if (!subset_map.empty()) {
+      Extend extend(*this, subset_map);
+      root->perform(&extend);
+    }
+
+    Remove_Placeholders remove_placeholders(*this);
+    root->perform(&remove_placeholders);
+
+    return root;
+  }
+
+  Block* Context::parse_string()
+  {
+    if (!source_c_str) return 0;
+    queue.clear();
+    if(is_indented_syntax_src) {
+      char * contents = sass2scss(source_c_str, SASS2SCSS_PRETTIFY_1);
+      add_source(input_path, input_path, contents);
+      return parse_file();
+    }
+    add_source(input_path, input_path, strdup(source_c_str));
+    return parse_file();
+  }
+
+  char* Context::compile_file()
+  {
+    // returns NULL if something fails
+    return compile_block(parse_file());
+  }
+
+  char* Context::compile_string()
+  {
+    // returns NULL if something fails
+    return compile_block(parse_string());
+  }
+
   string Context::format_source_mapping_url(const string& file)
   {
     string url = resolve_relative_path(file, output_path, cwd);
@@ -278,7 +328,7 @@ namespace Sass {
       ostringstream buffer;
       base64::encoder E;
       E.encode(is, buffer);
-      url = "data:text/css;base64," + buffer.str();
+      url = "data:application/json;base64," + buffer.str();
       url.erase(url.size() - 1);
     }
     return "/*# sourceMappingURL=" + url + " */";
@@ -293,40 +343,20 @@ namespace Sass {
     return result;
   }
 
-  // allow to optionally overwrite the input path
-  // default argument for input_path is string("stdin")
-  // usefull to influence the source-map generating etc.
-  char* Context::compile_string(const string& input_path)
-  {
-    if (!source_c_str) return 0;
-    queue.clear();
-    if(is_indented_syntax_src) {
-      char * contents = sass2scss(source_c_str, SASS2SCSS_PRETTIFY_1);
-      add_source(input_path, input_path, contents);
-      return compile_file();
-    }
-    add_source(input_path, input_path, strdup(source_c_str));
-    return compile_file();
-  }
 
-  std::vector<std::string> Context::get_included_files()
+  std::vector<std::string> Context::get_included_files(size_t skip)
   {
-      std::sort(included_files.begin(), included_files.end());
-      included_files.erase( std::unique( included_files.begin(), included_files.end() ), included_files.end());
-      return included_files;
+      vector<string> includes = included_files;
+      std::sort( includes.begin() + skip, includes.end() );
+      includes.erase( std::unique( includes.begin(), includes.end() ), includes.end() );
+      // the skip solution seems more robust, as we may have real files named stdin
+      // includes.erase( std::remove( includes.begin(), includes.end(), "stdin" ), includes.end() );
+      return includes;
   }
 
   string Context::get_cwd()
   {
-    const size_t wd_len = 1024;
-    char wd[wd_len];
-    string cwd = getcwd(wd, wd_len);
-#ifdef _WIN32
-    //convert backslashes to forward slashes
-    replace(cwd.begin(), cwd.end(), '\\', '/');
-#endif
-    if (cwd[cwd.length() - 1] != '/') cwd += '/';
-    return cwd;
+    return Sass::File::get_cwd();
   }
 
   void register_function(Context& ctx, Signature sig, Native_Function f, Env* env)
@@ -347,8 +377,7 @@ namespace Sass {
 
   void register_overload_stub(Context& ctx, string name, Env* env)
   {
-    Definition* stub = new (ctx.mem) Definition("[built-in function]",
-                                            Position(),
+    Definition* stub = new (ctx.mem) Definition(ParserState("[built-in function]"),
                                             0,
                                             name,
                                             0,
@@ -413,6 +442,7 @@ namespace Sass {
     register_function(ctx, abs_sig, abs, env);
     register_function(ctx, min_sig, min, env);
     register_function(ctx, max_sig, max, env);
+    register_function(ctx, random_sig, random, env);
     // List Functions
     register_function(ctx, length_sig, length, env);
     register_function(ctx, nth_sig, nth, env);
@@ -440,24 +470,33 @@ namespace Sass {
     register_function(ctx, global_variable_exists_sig, global_variable_exists, env);
     register_function(ctx, function_exists_sig, function_exists, env);
     register_function(ctx, mixin_exists_sig, mixin_exists, env);
+    register_function(ctx, feature_exists_sig, feature_exists, env);
     register_function(ctx, call_sig, call, env);
     // Boolean Functions
     register_function(ctx, not_sig, sass_not, env);
     register_function(ctx, if_sig, sass_if, env);
     // Path Functions
     register_function(ctx, image_url_sig, image_url, env);
+    // Misc Functions
+    register_function(ctx, inspect_sig, inspect, env);
+    register_function(ctx, unique_id_sig, unique_id, env);
   }
 
-  void register_c_functions(Context& ctx, Env* env, Sass_C_Function_Descriptor* descrs)
+  void register_c_functions(Context& ctx, Env* env, Sass_C_Function_List descrs)
   {
-    while (descrs->signature && descrs->function) {
+    while (descrs && *descrs) {
       register_c_function(ctx, env, *descrs);
       ++descrs;
     }
   }
-  void register_c_function(Context& ctx, Env* env, Sass_C_Function_Descriptor descr)
+  void register_c_function(Context& ctx, Env* env, Sass_C_Function_Callback descr)
   {
-    Definition* def = make_c_function(descr.signature, descr.function, descr.cookie, ctx);
+    Definition* def = make_c_function(
+      sass_function_get_signature(descr),
+      sass_function_get_function(descr),
+      sass_function_get_cookie(descr),
+      ctx
+    );
     def->environment(env);
     (*env)[def->name() + "[f]"] = def;
   }
