@@ -1,22 +1,12 @@
-#define SASS_EVAL
+#ifndef SASS_EVAL_H
+#define SASS_EVAL_H
 
 #include <iostream>
 
-#ifndef SASS_OPERATION
-#include "operation.hpp"
-#endif
-
-#ifndef SASS_ENVIRONMENT
-#include "environment.hpp"
-#endif
-
-#ifndef SASS
-#include "sass.h"
-#endif
-
-#ifndef SASS_POSITION
 #include "position.hpp"
-#endif
+#include "operation.hpp"
+#include "environment.hpp"
+#include "sass_values.h"
 
 namespace Sass {
   using namespace std;
@@ -48,6 +38,8 @@ namespace Sass {
     Expression* operator()(While*);
     Expression* operator()(Return*);
     Expression* operator()(Warning*);
+    Expression* operator()(Error*);
+    Expression* operator()(Debug*);
 
     Expression* operator()(List*);
     Expression* operator()(Map*);
@@ -63,7 +55,7 @@ namespace Sass {
     Expression* operator()(String_Constant*);
     Expression* operator()(Media_Query*);
     Expression* operator()(Media_Query_Expression*);
-    Expression* operator()(Feature_Queries*);
+    Expression* operator()(At_Root_Expression*);
     Expression* operator()(Feature_Query*);
     Expression* operator()(Feature_Query_Condition*);
     Expression* operator()(Null*);
@@ -74,8 +66,10 @@ namespace Sass {
     Expression* fallback(U x) { return fallback_impl(x); }
   };
 
-  Expression* cval_to_astnode(Sass_Value v, Context& ctx, Backtrace* backtrace, string path = "", Position position = Position());
+  Expression* cval_to_astnode(Sass_Value* v, Context& ctx, Backtrace* backtrace, ParserState pstate = ParserState("[AST]"));
 
   bool eq(Expression*, Expression*, Context&);
   bool lt(Expression*, Expression*, Context&);
 }
+
+#endif
