@@ -1,20 +1,13 @@
-#define SASS_EXPAND
+#ifndef SASS_EXPAND_H
+#define SASS_EXPAND_H
 
-#include <vector>
 #include <map>
+#include <vector>
 #include <iostream>
 
-#ifndef SASS_AST
 #include "ast.hpp"
-#endif
-
-#ifndef SASS_OPERATION
 #include "operation.hpp"
-#endif
-
-#ifndef SASS_ENVIRONMENT
 #include "environment.hpp"
-#endif
 
 namespace Sass {
   using namespace std;
@@ -34,6 +27,8 @@ namespace Sass {
     vector<Block*>    block_stack;
     vector<String*>   property_stack;
     vector<Selector*> selector_stack;
+    vector<Selector*> at_root_selector_stack;
+    bool              in_at_root;
     Backtrace*        backtrace;
 
     Statement* fallback_impl(AST_Node* n);
@@ -49,12 +44,15 @@ namespace Sass {
     Statement* operator()(Propset*);
     Statement* operator()(Media_Block*);
     Statement* operator()(Feature_Block*);
+    Statement* operator()(At_Root_Block*);
     Statement* operator()(At_Rule*);
     Statement* operator()(Declaration*);
     Statement* operator()(Assignment*);
     Statement* operator()(Import*);
     Statement* operator()(Import_Stub*);
     Statement* operator()(Warning*);
+    Statement* operator()(Error*);
+    Statement* operator()(Debug*);
     Statement* operator()(Comment*);
     Statement* operator()(If*);
     Statement* operator()(For*);
@@ -73,3 +71,5 @@ namespace Sass {
   };
 
 }
+
+#endif
