@@ -1,9 +1,13 @@
+#ifndef SASS_FILE_H
+#define SASS_FILE_H
+
 #include <string>
 
 namespace Sass {
   using namespace std;
   struct Context;
   namespace File {
+    string get_cwd();
     string base_name(string);
     string dir_name(string);
     string join_paths(string, string);
@@ -15,3 +19,5 @@ namespace Sass {
     char* read_file(string path);
   }
 }
+
+#endif
