@@ -2,6 +2,7 @@
 #include "ast.hpp"
 #include "context.hpp"
 #include <cmath>
+#include <string>
 #include <iostream>
 #include <iomanip>
 
@@ -15,19 +16,19 @@ namespace Sass {
   void Inspect::operator()(Block* block)
   {
     if (!block->is_root()) {
-      append_to_buffer(" {\n");
-      ++indentation;
+      append_to_buffer(" {" + ctx->linefeed);
+      ++ indentation;
     }
     for (size_t i = 0, L = block->length(); i < L; ++i) {
-      indent();
+      append_indent_to_buffer();
       (*block)[i]->perform(this);
       // extra newline at the end of top-level statements
-      if (block->is_root()) append_to_buffer("\n");
-      append_to_buffer("\n");
+      if (block->is_root()) append_to_buffer(ctx->linefeed);
+      append_to_buffer(ctx->linefeed);
     }
     if (!block->is_root()) {
-      --indentation;
-      indent();
+      -- indentation;
+      append_indent_to_buffer();
       append_to_buffer("}");
     }
     // remove extra newline that gets added after the last top-level block
@@ -36,6 +37,7 @@ namespace Sass {
       if (l > 2 && buffer[l-1] == '\n' && buffer[l-2] == '\n') {
         buffer.erase(l-1);
         if (ctx) ctx->source_map.remove_line();
+        source_map.remove_line();
       }
     }
   }
@@ -53,22 +55,34 @@ namespace Sass {
     propset->block()->perform(this);
   }
 
+  void Inspect::operator()(Bubble* bubble)
+  {
+    append_to_buffer("Bubble ( ");
+    bubble->node()->perform(this);
+    append_to_buffer(" )");
+  }
+
   void Inspect::operator()(Media_Block* media_block)
   {
-    if (ctx) ctx->source_map.add_mapping(media_block);
-    append_to_buffer("@media ");
+    append_to_buffer("@media", media_block, " ");
     media_block->media_queries()->perform(this);
     media_block->block()->perform(this);
   }
 
   void Inspect::operator()(Feature_Block* feature_block)
   {
-    if (ctx) ctx->source_map.add_mapping(feature_block);
-    append_to_buffer("@supports ");
+    append_to_buffer("@supports", feature_block, " ");
     feature_block->feature_queries()->perform(this);
     feature_block->block()->perform(this);
   }
 
+  void Inspect::operator()(At_Root_Block* at_root_block)
+  {
+    append_to_buffer("@at-root ", at_root_block, " ");
+    if(at_root_block->expression()) at_root_block->expression()->perform(this);
+    at_root_block->block()->perform(this);
+  }
+
   void Inspect::operator()(At_Rule* at_rule)
   {
     append_to_buffer(at_rule->keyword());
@@ -87,12 +101,18 @@ namespace Sass {
   void Inspect::operator()(Declaration* dec)
   {
     if (dec->value()->concrete_type() == Expression::NULL_VAL) return;
-    if (ctx) ctx->source_map.add_mapping(dec->property());
+    if (ctx) ctx->source_map.add_open_mapping(dec->property());
+    source_map.add_open_mapping(dec->property());
     dec->property()->perform(this);
+    if (ctx) ctx->source_map.add_close_mapping(dec->property());
+    source_map.add_close_mapping(dec->property());
     append_to_buffer(": ");
-    if (ctx) ctx->source_map.add_mapping(dec->value());
+    if (ctx) ctx->source_map.add_open_mapping(dec->value());
+    source_map.add_open_mapping(dec->value());
     dec->value()->perform(this);
     if (dec->is_important()) append_to_buffer(" !important");
+    if (ctx) ctx->source_map.add_close_mapping(dec->value());
+    source_map.add_close_mapping(dec->value());
     append_to_buffer(";");
   }
 
@@ -108,14 +128,12 @@ namespace Sass {
   void Inspect::operator()(Import* import)
   {
     if (!import->urls().empty()) {
-      if (ctx) ctx->source_map.add_mapping(import);
-      append_to_buffer("@import ");
+      append_to_buffer("@import", import, " ");
       import->urls().front()->perform(this);
       append_to_buffer(";");
       for (size_t i = 1, S = import->urls().size(); i < S; ++i) {
-        append_to_buffer("\n");
-        if (ctx) ctx->source_map.add_mapping(import);
-        append_to_buffer("@import ");
+        append_to_buffer(ctx->linefeed);
+        append_to_buffer("@import", import, " ");
         import->urls()[i]->perform(this);
         append_to_buffer(";");
       }
@@ -124,20 +142,32 @@ namespace Sass {
 
   void Inspect::operator()(Import_Stub* import)
   {
-    if (ctx) ctx->source_map.add_mapping(import);
-    append_to_buffer("@import ");
+    append_to_buffer("@import", import, " ");
     append_to_buffer(import->file_name());
     append_to_buffer(";");
   }
 
   void Inspect::operator()(Warning* warning)
   {
-    if (ctx) ctx->source_map.add_mapping(warning);
-    append_to_buffer("@warn ");
+    append_to_buffer("@warn", warning, " ");
     warning->message()->perform(this);
     append_to_buffer(";");
   }
 
+  void Inspect::operator()(Error* error)
+  {
+    append_to_buffer("@error", error, " ");
+    error->message()->perform(this);
+    append_to_buffer(";");
+  }
+
+  void Inspect::operator()(Debug* debug)
+  {
+    append_to_buffer("@debug", debug, " ");
+    debug->value()->perform(this);
+    append_to_buffer(";");
+  }
+
   void Inspect::operator()(Comment* comment)
   {
     comment->text()->perform(this);
@@ -145,12 +175,12 @@ namespace Sass {
 
   void Inspect::operator()(If* cond)
   {
-    append_to_buffer("@if ");
+    append_to_buffer("@if", cond, " ");
     cond->predicate()->perform(this);
     cond->consequent()->perform(this);
     if (cond->alternative()) {
-      append_to_buffer("\n");
-      indent();
+      append_to_buffer(ctx->linefeed);
+      append_indent_to_buffer();
       append_to_buffer("else");
       cond->alternative()->perform(this);
     }
@@ -158,7 +188,7 @@ namespace Sass {
 
   void Inspect::operator()(For* loop)
   {
-    append_to_buffer("@for ");
+    append_to_buffer("@for", loop, " ");
     append_to_buffer(loop->variable());
     append_to_buffer(" from ");
     loop->lower_bound()->perform(this);
@@ -169,7 +199,7 @@ namespace Sass {
 
   void Inspect::operator()(Each* loop)
   {
-    append_to_buffer("@each ");
+    append_to_buffer("@each", loop, " ");
     append_to_buffer(loop->variables()[0]);
     for (size_t i = 1, L = loop->variables().size(); i < L; ++i) {
       append_to_buffer(", ");
@@ -182,21 +212,21 @@ namespace Sass {
 
   void Inspect::operator()(While* loop)
   {
-    append_to_buffer("@while ");
+    append_to_buffer("@while", loop, " ");
     loop->predicate()->perform(this);
     loop->block()->perform(this);
   }
 
   void Inspect::operator()(Return* ret)
   {
-    append_to_buffer("@return ");
+    append_to_buffer("@return", ret, " ");
     ret->value()->perform(this);
     append_to_buffer(";");
   }
 
   void Inspect::operator()(Extension* extend)
   {
-    append_to_buffer("@extend ");
+    append_to_buffer("@extend", extend, " ");
     extend->selector()->perform(this);
     append_to_buffer(";");
   }
@@ -204,9 +234,9 @@ namespace Sass {
   void Inspect::operator()(Definition* def)
   {
     if (def->type() == Definition::MIXIN) {
-      append_to_buffer("@mixin ");
+      append_to_buffer("@mixin", def, " ");
     } else {
-      append_to_buffer("@function ");
+      append_to_buffer("@function", def, " ");
     }
     append_to_buffer(def->name());
     def->parameters()->perform(this);
@@ -215,7 +245,8 @@ namespace Sass {
 
   void Inspect::operator()(Mixin_Call* call)
   {
-    append_to_buffer(string("@include ") += call->name());
+    append_to_buffer("@include", call, " ");
+    append_to_buffer(call->name());
     if (call->arguments()) {
       call->arguments()->perform(this);
     }
@@ -228,8 +259,25 @@ namespace Sass {
 
   void Inspect::operator()(Content* content)
   {
-    if (ctx) ctx->source_map.add_mapping(content);
-    append_to_buffer("@content;");
+    append_to_buffer("@content", content, ";");
+  }
+
+  void Inspect::operator()(Map* map)
+  {
+    if (map->empty()) return;
+    if (map->is_invisible()) return;
+    bool items_output = false;
+    append_to_buffer("(");
+    for (auto key : map->keys()) {
+      if (key->is_invisible()) continue;
+      if (map->at(key)->is_invisible()) continue;
+      if (items_output) append_to_buffer(", ");
+      key->perform(this);
+      append_to_buffer(": ");
+      map->at(key)->perform(this);
+      items_output = true;
+    }
+    append_to_buffer(")");
   }
 
   void Inspect::operator()(List* list)
@@ -326,14 +374,32 @@ namespace Sass {
     ss.precision(ctx ? ctx->precision : 5);
     ss << fixed << n->value();
     string d(ss.str());
+    // store if the value did not equal zero
+    // if after applying precsision, the value gets
+    // truncated to zero, sass emits 0.0 instead of 0
+    bool nonzero = n->value() != 0;
     for (size_t i = d.length()-1; d[i] == '0'; --i) {
       d.resize(d.length()-1);
     }
     if (d[d.length()-1] == '.') d.resize(d.length()-1);
-    if (n->numerator_units().size() > 1 || n->denominator_units().size() > 0) {
-      error(d + n->unit() + " is not a valid CSS value", n->path(), n->position());
+    if (n->numerator_units().size() > 1 ||
+        n->denominator_units().size() > 0 ||
+        (n->numerator_units().size() && n->numerator_units()[0].find_first_of('/') != string::npos) ||
+        (n->numerator_units().size() && n->numerator_units()[0].find_first_of('*') != string::npos)
+    ) {
+      error(d + n->unit() + " isn't a valid CSS value.", n->pstate());
     }
-    append_to_buffer(d == "-0" ? "0" : d);
+    if (!n->zero()) {
+      if (d.substr(0, 3) == "-0.") d.erase(1, 1);
+      if (d.substr(0, 2) == "0.") d.erase(0, 1);
+    }
+    // remove the leading minus
+    if (d == "-0") d.erase(0, 1);
+    // use fractional output if we had
+    // a value before it got truncated
+    if (d == "0" && nonzero) d = "0.0";
+    // append number and unit
+    append_to_buffer(d);
     append_to_buffer(n->unit());
   }
 
@@ -357,6 +423,9 @@ namespace Sass {
     if (!c->disp().empty()) {
       ss << c->disp();
     }
+    else if (r == 0 && g == 0 && b == 0 && a == 0) {
+        ss << "transparent";
+    }
     else if (a >= 1) {
       // see if it's a named color
       int numval = r * 0x10000;
@@ -401,16 +470,7 @@ namespace Sass {
 
   void Inspect::operator()(String_Constant* s)
   {
-    append_to_buffer(s->needs_unquoting() ? unquote(s->value()) : s->value());
-  }
-
-  void Inspect::operator()(Feature_Queries* fq)
-  {
-    size_t i = 0;
-    (*fq)[i++]->perform(this);
-    for (size_t L = fq->length(); i < L; ++i) {
-      (*fq)[i]->perform(this);
-    }
+    append_to_buffer(s->needs_unquoting() ? unquote(s->value()) : s->value(), s);
   }
 
   void Inspect::operator()(Feature_Query* fq)
@@ -418,7 +478,6 @@ namespace Sass {
     size_t i = 0;
     (*fq)[i++]->perform(this);
     for (size_t L = fq->length(); i < L; ++i) {
-      if (fq->is_negated()) append_to_buffer("not ");
       (*fq)[i]->perform(this);
     }
   }
@@ -429,16 +488,21 @@ namespace Sass {
       append_to_buffer(" and ");
     else if (fqc->operand() == Feature_Query_Condition::OR)
       append_to_buffer(" or ");
+    else if (fqc->operand() == Feature_Query_Condition::NOT)
+      append_to_buffer(" not ");
 
-    if (fqc->is_negated()) append_to_buffer("not ");
+    if (!fqc->is_root()) append_to_buffer("(");
 
-    append_to_buffer("(");
-    fqc->feature()->perform(this);
-    if (fqc->value()) {
+    if (!fqc->length()) {
+      fqc->feature()->perform(this);
       append_to_buffer(": ");
       fqc->value()->perform(this);
     }
-    append_to_buffer(")");
+    // else
+    for (size_t i = 0, L = fqc->length(); i < L; ++i)
+      (*fqc)[i]->perform(this);
+
+    if (!fqc->is_root()) append_to_buffer(")");
   }
 
   void Inspect::operator()(Media_Query* mq)
@@ -461,14 +525,42 @@ namespace Sass {
   void Inspect::operator()(Media_Query_Expression* mqe)
   {
     if (mqe->is_interpolated()) {
+      if (ctx) ctx->source_map.add_open_mapping(mqe->feature());
+      source_map.add_open_mapping(mqe->feature());
       mqe->feature()->perform(this);
+      if (ctx) ctx->source_map.add_close_mapping(mqe->feature());
+      source_map.add_close_mapping(mqe->feature());
     }
     else {
       append_to_buffer("(");
+      if (ctx) ctx->source_map.add_open_mapping(mqe->feature());
+      source_map.add_open_mapping(mqe->feature());
       mqe->feature()->perform(this);
+      if (ctx) ctx->source_map.add_close_mapping(mqe->feature());
+      source_map.add_close_mapping(mqe->feature());
       if (mqe->value()) {
         append_to_buffer(": ");
+        if (ctx) ctx->source_map.add_open_mapping(mqe->value());
+        source_map.add_open_mapping(mqe->value());
         mqe->value()->perform(this);
+        if (ctx) ctx->source_map.add_close_mapping(mqe->value());
+        source_map.add_close_mapping(mqe->value());
+      }
+      append_to_buffer(")");
+    }
+  }
+
+  void Inspect::operator()(At_Root_Expression* ae)
+  {
+    if (ae->is_interpolated()) {
+      ae->feature()->perform(this);
+    }
+    else {
+      append_to_buffer("(");
+      ae->feature()->perform(this);
+      if (ae->value()) {
+        append_to_buffer(": ");
+        ae->value()->perform(this);
       }
       append_to_buffer(")");
     }
@@ -515,7 +607,11 @@ namespace Sass {
     if (a->value()->concrete_type() == Expression::NULL_VAL) {
       return;
     }
-    a->value()->perform(this);
+    if (a->value()->concrete_type() == Expression::STRING) {
+      String_Constant* s = static_cast<String_Constant*>(a->value());
+      if (s->is_quoted()) s->value(quote(unquote(s->value()), String_Constant::double_quote()));
+      s->perform(this);
+    } else a->value()->perform(this);
     if (a->is_rest_argument()) {
       append_to_buffer("...");
     }
@@ -548,25 +644,24 @@ namespace Sass {
 
   void Inspect::operator()(Selector_Placeholder* s)
   {
-    append_to_buffer(s->name());
+    append_to_buffer(s->name(), s);
   }
 
   void Inspect::operator()(Type_Selector* s)
   {
-    if (ctx) ctx->source_map.add_mapping(s);
-    append_to_buffer(s->name());
+    append_to_buffer(s->name(), s);
   }
 
   void Inspect::operator()(Selector_Qualifier* s)
   {
-    if (ctx) ctx->source_map.add_mapping(s);
-    append_to_buffer(s->name());
+    append_to_buffer(s->name(), s);
   }
 
   void Inspect::operator()(Attribute_Selector* s)
   {
-    if (ctx) ctx->source_map.add_mapping(s);
     append_to_buffer("[");
+    if (ctx) ctx->source_map.add_open_mapping(s);
+    source_map.add_open_mapping(s);
     append_to_buffer(s->name());
     if (!s->matcher().empty()) {
       append_to_buffer(s->matcher());
@@ -575,13 +670,14 @@ namespace Sass {
       }
       // append_to_buffer(s->value());
     }
+    if (ctx) ctx->source_map.add_close_mapping(s);
+    source_map.add_close_mapping(s);
     append_to_buffer("]");
   }
 
   void Inspect::operator()(Pseudo_Selector* s)
   {
-    if (ctx) ctx->source_map.add_mapping(s);
-    append_to_buffer(s->name());
+    append_to_buffer(s->name(), s);
     if (s->expression()) {
       s->expression()->perform(this);
       append_to_buffer(")");
@@ -590,8 +686,7 @@ namespace Sass {
 
   void Inspect::operator()(Wrapped_Selector* s)
   {
-    if (ctx) ctx->source_map.add_mapping(s);
-    append_to_buffer(s->name());
+    append_to_buffer(s->name(), s);
     s->selector()->perform(this);
     append_to_buffer(")");
   }
@@ -625,18 +720,31 @@ namespace Sass {
   void Inspect::operator()(Selector_List* g)
   {
     if (g->empty()) return;
+    if (ctx) ctx->source_map.add_open_mapping((*g)[0]);
+    source_map.add_open_mapping((*g)[0]);
     (*g)[0]->perform(this);
+    if (ctx) ctx->source_map.add_close_mapping((*g)[0]);
+    source_map.add_close_mapping((*g)[0]);
     for (size_t i = 1, L = g->length(); i < L; ++i) {
       append_to_buffer(", ");
+      if (ctx) ctx->source_map.add_open_mapping((*g)[i]);
+      source_map.add_open_mapping((*g)[i]);
       (*g)[i]->perform(this);
+      if (ctx) ctx->source_map.add_close_mapping((*g)[i]);
+      source_map.add_close_mapping((*g)[i]);
     }
   }
 
   inline void Inspect::fallback_impl(AST_Node* n)
   { }
 
-  void Inspect::indent()
-  { append_to_buffer(string(2*indentation, ' ')); }
+  void Inspect::append_indent_to_buffer()
+  {
+    string indent = "";
+    for (size_t i = 0; i < indentation; i++)
+      indent += ctx->indent;
+    append_to_buffer(indent);
+  }
 
   string unquote(const string& s)
   {
@@ -652,7 +760,7 @@ namespace Sass {
     t.reserve(s.length()-2);
     for (size_t i = 1, L = s.length()-1; i < L; ++i) {
       // if we see a quote, we need to remove the preceding backslash from t
-      if (s[i] == q) t.erase(t.length()-1);
+      if (s[i-1] == '\\' && s[i] == q) t.erase(t.length()-1);
       t.push_back(s[i]);
     }
     return t;
@@ -676,8 +784,23 @@ namespace Sass {
   void Inspect::append_to_buffer(const string& text)
   {
     buffer += text;
-    if (ctx && !ctx->_skip_source_map_update)
-      ctx->source_map.update_column(text);
+    if (ctx) ctx->source_map.update_column(text);
+    source_map.update_column(text);
+  }
+
+  void Inspect::append_to_buffer(const string& text, AST_Node* node)
+  {
+    if (ctx) ctx->source_map.add_open_mapping(node);
+    source_map.add_open_mapping(node);
+    append_to_buffer(text);
+    if (ctx) ctx->source_map.add_close_mapping(node);
+    source_map.add_close_mapping(node);
+  }
+
+  void Inspect::append_to_buffer(const string& text, AST_Node* node, const string& tail)
+  {
+    append_to_buffer(text, node);
+    append_to_buffer(tail);
   }
 
 }
