@@ -1,15 +1,11 @@
-#ifndef SASS_INSPECT
-#define SASS_INSPECT
+#ifndef SASS_INSPECT_H
+#define SASS_INSPECT_H
 
 #include <string>
 
-#ifndef SASS_OPERATION
+#include "position.hpp"
 #include "operation.hpp"
-#endif
-
-// #ifndef SASS_TO_STRING
-// #include "to_string.hpp"
-// #endif
+#include "source_map.hpp"
 
 namespace Sass {
   using namespace std;
@@ -23,14 +19,18 @@ namespace Sass {
     string buffer;
     size_t indentation;
     Context* ctx;
-    void indent();
 
     void fallback_impl(AST_Node* n);
 
+  public:
+    void append_indent_to_buffer();
     void append_to_buffer(const string& text);
+    void append_to_buffer(const string& text, AST_Node* node);
+    void append_to_buffer(const string& text, AST_Node* node, const string& tail);
 
   public:
 
+    SourceMap source_map;
     Inspect(Context* ctx = 0);
     virtual ~Inspect();
 
@@ -40,14 +40,18 @@ namespace Sass {
     virtual void operator()(Block*);
     virtual void operator()(Ruleset*);
     virtual void operator()(Propset*);
+    virtual void operator()(Bubble*);
     virtual void operator()(Feature_Block*);
     virtual void operator()(Media_Block*);
+    virtual void operator()(At_Root_Block*);
     virtual void operator()(At_Rule*);
     virtual void operator()(Declaration*);
     virtual void operator()(Assignment*);
     virtual void operator()(Import*);
     virtual void operator()(Import_Stub*);
     virtual void operator()(Warning*);
+    virtual void operator()(Error*);
+    virtual void operator()(Debug*);
     virtual void operator()(Comment*);
     virtual void operator()(If*);
     virtual void operator()(For*);
@@ -59,6 +63,7 @@ namespace Sass {
     virtual void operator()(Mixin_Call*);
     virtual void operator()(Content*);
     // expressions
+    virtual void operator()(Map*);
     virtual void operator()(List*);
     virtual void operator()(Binary_Expression*);
     virtual void operator()(Unary_Expression*);
@@ -71,11 +76,11 @@ namespace Sass {
     virtual void operator()(Boolean*);
     virtual void operator()(String_Schema*);
     virtual void operator()(String_Constant*);
-    virtual void operator()(Feature_Queries*);
     virtual void operator()(Feature_Query*);
     virtual void operator()(Feature_Query_Condition*);
     virtual void operator()(Media_Query*);
     virtual void operator()(Media_Query_Expression*);
+    virtual void operator()(At_Root_Expression*);
     virtual void operator()(Null*);
     // parameters and arguments
     virtual void operator()(Parameter*);
@@ -96,7 +101,7 @@ namespace Sass {
     virtual void operator()(Selector_List*);
 
     template <typename U>
-    void fallback(U x) { fallback_impl(x); }
+    void fallback(U x) { fallback_impl(reinterpret_cast<AST_Node*>(x)); }
   };
 
   string unquote(const string&);
