@@ -1,12 +1,13 @@
-#define SASS_OPERATION
+#ifndef SASS_OPERATION_H
+#define SASS_OPERATION_H
+
+#include <iostream>
+#include <typeinfo>
 
 #include "ast_fwd_decl.hpp"
 
-#include <iostream>
-using namespace std;
-#include <typeinfo>
-
 namespace Sass {
+  using namespace std;
 
   template<typename T>
   class Operation {
@@ -17,14 +18,18 @@ namespace Sass {
     virtual T operator()(Block* x)                  = 0;
     virtual T operator()(Ruleset* x)                = 0;
     virtual T operator()(Propset* x)                = 0;
+    virtual T operator()(Bubble* x)                 = 0;
     virtual T operator()(Feature_Block* x)          = 0;
     virtual T operator()(Media_Block* x)            = 0;
+    virtual T operator()(At_Root_Block* x)          = 0;
     virtual T operator()(At_Rule* x)                = 0;
     virtual T operator()(Declaration* x)            = 0;
     virtual T operator()(Assignment* x)             = 0;
     virtual T operator()(Import* x)                 = 0;
     virtual T operator()(Import_Stub* x)            = 0;
     virtual T operator()(Warning* x)                = 0;
+    virtual T operator()(Error* x)                  = 0;
+    virtual T operator()(Debug* x)                  = 0;
     virtual T operator()(Comment* x)                = 0;
     virtual T operator()(If* x)                     = 0;
     virtual T operator()(For* x)                    = 0;
@@ -49,11 +54,11 @@ namespace Sass {
     virtual T operator()(Boolean* x)                = 0;
     virtual T operator()(String_Schema* x)          = 0;
     virtual T operator()(String_Constant* x)        = 0;
-    virtual T operator()(Feature_Queries* x)        = 0;
     virtual T operator()(Feature_Query* x)          = 0;
     virtual T operator()(Feature_Query_Condition* x)= 0;
     virtual T operator()(Media_Query* x)            = 0;
     virtual T operator()(Media_Query_Expression* x) = 0;
+    virtual T operator()(At_Root_Expression* x)     = 0;
     virtual T operator()(Null* x)                   = 0;
     // parameters and arguments
     virtual T operator()(Parameter* x)              = 0;
@@ -86,14 +91,18 @@ namespace Sass {
     virtual T operator()(Block* x)                  { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Ruleset* x)                { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Propset* x)                { return static_cast<D*>(this)->fallback(x); }
+    virtual T operator()(Bubble* x)                 { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Feature_Block* x)          { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Media_Block* x)            { return static_cast<D*>(this)->fallback(x); }
+    virtual T operator()(At_Root_Block* x)          { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(At_Rule* x)                { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Declaration* x)            { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Assignment* x)             { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Import* x)                 { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Import_Stub* x)            { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Warning* x)                { return static_cast<D*>(this)->fallback(x); }
+    virtual T operator()(Error* x)                  { return static_cast<D*>(this)->fallback(x); }
+    virtual T operator()(Debug* x)                  { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Comment* x)                { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(If* x)                     { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(For* x)                    { return static_cast<D*>(this)->fallback(x); }
@@ -118,11 +127,11 @@ namespace Sass {
     virtual T operator()(Boolean* x)                { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(String_Schema* x)          { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(String_Constant* x)        { return static_cast<D*>(this)->fallback(x); }
-    virtual T operator()(Feature_Queries* x)        { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Feature_Query* x)          { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Feature_Query_Condition* x){ return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Media_Query* x)            { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Media_Query_Expression* x) { return static_cast<D*>(this)->fallback(x); }
+    virtual T operator()(At_Root_Expression* x)     { return static_cast<D*>(this)->fallback(x); }
     virtual T operator()(Null* x)                   { return static_cast<D*>(this)->fallback(x); }
     // parameters and arguments
     virtual T operator()(Parameter* x)              { return static_cast<D*>(this)->fallback(x); }
@@ -149,3 +158,5 @@ namespace Sass {
   inline Operation_CRTP<T, D>::~Operation_CRTP()    { }
 
 }
+
+#endif
