@@ -1,30 +1,32 @@
-#include "output_compressed.hpp"
-#include "inspect.hpp"
-#include "ast.hpp"
-#include "context.hpp"
-#include "to_string.hpp"
-#include "util.hpp"
 #include <cmath>
 #include <iomanip>
 
+#include "ast.hpp"
+#include "util.hpp"
+#include "context.hpp"
+#include "inspect.hpp"
+#include "to_string.hpp"
+#include "output_compressed.hpp"
+
 namespace Sass {
   using namespace std;
 
-  Output_Compressed::Output_Compressed(Context* ctx) : buffer(""), rendered_imports(""), ctx(ctx) { }
+  Output_Compressed::Output_Compressed(Context* ctx) : Output(ctx), seen_utf8(false) { }
   Output_Compressed::~Output_Compressed() { }
 
   inline void Output_Compressed::fallback_impl(AST_Node* n)
   {
     Inspect i(ctx);
     n->perform(&i);
-    buffer += i.get_buffer();
+    append_to_buffer(i.get_buffer());
   }
 
   void Output_Compressed::operator()(Import* imp)
   {
-    Inspect insp(ctx);
-    imp->perform(&insp);
-    rendered_imports += insp.get_buffer();
+    // Inspect insp(ctx);
+    // imp->perform(&insp);
+    // rendered_imports += insp.get_buffer();
+    top_imports.push_back(imp);
   }
 
   void Output_Compressed::operator()(Block* b)
@@ -77,6 +79,8 @@ namespace Sass {
 
   void Output_Compressed::operator()(Media_Block* m)
   {
+    if (m->is_invisible()) return;
+
     List*  q     = m->media_queries();
     Block* b     = m->block();
 
@@ -91,8 +95,9 @@ namespace Sass {
       return;
     }
 
-    ctx->source_map.add_mapping(m);
+    ctx->source_map.add_open_mapping(m);
     append_singleline_part_to_buffer("@media ");
+    ctx->source_map.add_close_mapping(m);
     q->perform(this);
     append_singleline_part_to_buffer("{");
 
@@ -185,10 +190,10 @@ namespace Sass {
     }
     // Print if OK
     if(bPrintExpression) {
-      if (ctx) ctx->source_map.add_mapping(d->property());
+      if (ctx) ctx->source_map.add_open_mapping(d->property());
       d->property()->perform(this);
       append_singleline_part_to_buffer(":");
-      if (ctx) ctx->source_map.add_mapping(d->value());
+      if (ctx) ctx->source_map.add_open_mapping(d->value());
       d->value()->perform(this);
       if (d->is_important()) append_singleline_part_to_buffer("!important");
       append_singleline_part_to_buffer(";");
@@ -205,7 +210,7 @@ namespace Sass {
     else {
       Inspect i(ctx);
       c->perform(&i);
-      buffer += i.get_buffer();
+      append_to_buffer(i.get_buffer());
     }
   }
 
@@ -245,6 +250,9 @@ namespace Sass {
     if (!c->disp().empty()) {
       ss << c->disp();
     }
+    else if (r == 0 && g == 0 && b == 0 && a == 0) {
+        ss << "transparent";
+    }
     else if (a >= 1) {
       // see if it's a named color
       int numval = r * 0x10000;
@@ -360,9 +368,10 @@ namespace Sass {
 
   void Output_Compressed::append_singleline_part_to_buffer(const string& text)
   {
-    buffer += text;
-    if (ctx && !ctx->_skip_source_map_update)
-      ctx->source_map.update_column(text);
+    append_to_buffer(text);
   }
 
+  // compile output implementation
+  template class Output<Output_Compressed>;
+
 }
