@@ -1,18 +1,23 @@
-#include "output_nested.hpp"
-#include "inspect.hpp"
-#include "ast.hpp"
-#include "context.hpp"
-#include "to_string.hpp"
-#include "util.hpp"
 #include <iostream>
 #include <sstream>
 #include <typeinfo>
 
+#include "ast.hpp"
+#include "util.hpp"
+#include "context.hpp"
+#include "inspect.hpp"
+#include "to_string.hpp"
+#include "output_nested.hpp"
+
 namespace Sass {
   using namespace std;
 
   Output_Nested::Output_Nested(bool source_comments, Context* ctx)
-  : buffer(""), rendered_imports(""), indentation(0), source_comments(source_comments), ctx(ctx)
+  : Output(ctx),
+    indentation(0),
+    source_comments(source_comments),
+    in_directive(false),
+    in_keyframes(false)
   { }
   Output_Nested::~Output_Nested() { }
 
@@ -20,17 +25,15 @@ namespace Sass {
   {
     Inspect i(ctx);
     n->perform(&i);
-    buffer += i.get_buffer();
+    append_to_buffer(i.get_buffer());
   }
 
   void Output_Nested::operator()(Import* imp)
   {
-    Inspect insp(ctx);
-    imp->perform(&insp);
-    if (!rendered_imports.empty()) {
-      rendered_imports += "\n";
-    }
-    rendered_imports += insp.get_buffer();
+    // Inspect insp(ctx);
+    // imp->perform(&insp);
+    // insp.get_buffer();
+    top_imports.push_back(imp);
   }
 
   void Output_Nested::operator()(Block* b)
@@ -39,7 +42,20 @@ namespace Sass {
     for (size_t i = 0, L = b->length(); i < L; ++i) {
       size_t old_len = buffer.length();
       (*b)[i]->perform(this);
-      if (i < L-1 && old_len < buffer.length()) append_to_buffer("\n");
+      if (i < L-1 && old_len < buffer.length()) append_to_buffer(ctx->linefeed);
+    }
+  }
+
+  void Output_Nested::operator()(Comment* c)
+  {
+    To_String to_string;
+    string txt = c->text()->perform(&to_string);
+    if (buffer.size() + top_imports.size() == 0) {
+      top_comments.push_back(c);
+    } else {
+      Inspect i(ctx);
+      c->perform(&i);
+      append_to_buffer(i.get_buffer());
     }
   }
 
@@ -65,15 +81,16 @@ namespace Sass {
 
     if (b->has_non_hoistable()) {
       decls = true;
+      indentation += r->tabs();
       indent();
       if (source_comments) {
         stringstream ss;
-        ss << "/* line " << r->position().line << ", " << r->path() << " */" << endl;
+        ss << "/* line " << r->pstate().line+1 << ", " << r->pstate().path << " */" << endl;
         append_to_buffer(ss.str());
         indent();
       }
       s->perform(this);
-      append_to_buffer(" {\n");
+      append_to_buffer(" {" + ctx->linefeed);
       ++indentation;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
         Statement* stm = (*b)[i];
@@ -102,13 +119,21 @@ namespace Sass {
         if (!stm->is_hoistable() && bPrintExpression) {
           if (!stm->block()) indent();
           stm->perform(this);
-          append_to_buffer("\n");
+          if (i < L-1) append_to_buffer(ctx->linefeed);
         }
       }
       --indentation;
-      buffer.erase(buffer.length()-1);
-      if (ctx) ctx->source_map.remove_line();
-      append_to_buffer(" }\n");
+      indentation -= r->tabs();
+
+      while (buffer.substr(buffer.length()-ctx->linefeed.length()) == ctx->linefeed) {
+        buffer.erase(buffer.length()-1);
+        if (ctx) ctx->source_map.remove_line();
+      }
+
+      append_to_buffer(" }");
+      if (r->group_end()) append_to_buffer(ctx->linefeed);
+      // Match Sass 3.4.9 behaviour
+      if (in_directive && !in_keyframes) append_to_buffer(ctx->linefeed);
     }
 
     if (b->has_hoistable()) {
@@ -126,7 +151,9 @@ namespace Sass {
 
   void Output_Nested::operator()(Feature_Block* f)
   {
-    Feature_Queries*  q = f->feature_queries();
+    if (f->is_invisible()) return;
+
+    Feature_Query* q    = f->feature_queries();
     Block* b            = f->block();
 
     // Filter out feature blocks that aren't printable (process its children though)
@@ -140,11 +167,17 @@ namespace Sass {
       return;
     }
 
+    indentation += f->tabs();
     indent();
-    ctx->source_map.add_mapping(f);
-    append_to_buffer("@supports ");
+    append_to_buffer("@supports", f);
+    append_to_buffer(" ");
     q->perform(this);
-    append_to_buffer(" {\n");
+    append_to_buffer(" ");
+    append_to_buffer("{");
+    append_to_buffer(ctx->linefeed);
+
+    bool old_in_directive = in_directive;
+    in_directive = true;
 
     Selector* e = f->selector();
     if (e && b->has_non_hoistable()) {
@@ -152,7 +185,7 @@ namespace Sass {
       ++indentation;
       indent();
       e->perform(this);
-      append_to_buffer(" {\n");
+      append_to_buffer(" {" + ctx->linefeed);
 
       ++indentation;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
@@ -160,14 +193,16 @@ namespace Sass {
         if (!stm->is_hoistable()) {
           if (!stm->block()) indent();
           stm->perform(this);
-          append_to_buffer("\n");
+          if (i < L-1) append_to_buffer(ctx->linefeed);
         }
       }
       --indentation;
 
-      buffer.erase(buffer.length()-1);
-      if (ctx) ctx->source_map.remove_line();
-      append_to_buffer(" }\n");
+      in_directive = old_in_directive;
+
+      // buffer.erase(buffer.length()-1);
+      // if (ctx) ctx->source_map.remove_line();
+      append_to_buffer(" }" + ctx->linefeed);
       --indentation;
 
       ++indentation;
@@ -190,18 +225,28 @@ namespace Sass {
           if (!stm->block()) indent();
         }
         stm->perform(this);
-        append_to_buffer("\n");
+        if (!stm->is_hoistable()) append_to_buffer(ctx->linefeed);
       }
       --indentation;
     }
 
-    buffer.erase(buffer.length()-1);
-    if (ctx) ctx->source_map.remove_line();
-    append_to_buffer(" }\n");
+    while (buffer.substr(buffer.length()-ctx->linefeed.length()) == ctx->linefeed) {
+      buffer.erase(buffer.length()-1);
+      if (ctx) ctx->source_map.remove_line();
+    }
+
+    in_directive = old_in_directive;
+
+    append_to_buffer(" }");
+    if (f->group_end() || in_directive) append_to_buffer(ctx->linefeed);
+
+    indentation -= f->tabs();
   }
 
   void Output_Nested::operator()(Media_Block* m)
   {
+    if (m->is_invisible()) return;
+
     List*  q     = m->media_queries();
     Block* b     = m->block();
 
@@ -216,11 +261,17 @@ namespace Sass {
       return;
     }
 
+    indentation += m->tabs();
     indent();
-    ctx->source_map.add_mapping(m);
-    append_to_buffer("@media ");
+    append_to_buffer("@media", m);
+    append_to_buffer(" ");
     q->perform(this);
-    append_to_buffer(" {\n");
+    append_to_buffer(" ");
+    append_to_buffer("{");
+    append_to_buffer(ctx->linefeed);
+
+    bool old_in_directive = in_directive;
+    in_directive = true;
 
     Selector* e = m->selector();
     if (e && b->has_non_hoistable()) {
@@ -228,7 +279,7 @@ namespace Sass {
       ++indentation;
       indent();
       e->perform(this);
-      append_to_buffer(" {\n");
+      append_to_buffer(" {" + ctx->linefeed);
 
       ++indentation;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
@@ -236,14 +287,16 @@ namespace Sass {
         if (!stm->is_hoistable()) {
           if (!stm->block()) indent();
           stm->perform(this);
-          append_to_buffer("\n");
+          if (i < L-1) append_to_buffer(ctx->linefeed);
         }
       }
       --indentation;
 
-      buffer.erase(buffer.length()-1);
-      if (ctx) ctx->source_map.remove_line();
-      append_to_buffer(" }\n");
+      in_directive = old_in_directive;
+
+      // buffer.erase(buffer.length()-1);
+      // if (ctx) ctx->source_map.remove_line();
+      append_to_buffer(" }" + ctx->linefeed);
       --indentation;
 
       ++indentation;
@@ -266,14 +319,22 @@ namespace Sass {
           if (!stm->block()) indent();
         }
         stm->perform(this);
-        append_to_buffer("\n");
+        if (!stm->is_hoistable()) append_to_buffer(ctx->linefeed);
       }
       --indentation;
     }
 
-    buffer.erase(buffer.length()-1);
-    if (ctx) ctx->source_map.remove_line();
-    append_to_buffer(" }\n");
+    while (buffer.substr(buffer.length()-ctx->linefeed.length()) == ctx->linefeed) {
+      buffer.erase(buffer.length()-1);
+      if (ctx) ctx->source_map.remove_line();
+    }
+
+    in_directive = old_in_directive;
+
+    append_to_buffer(" }");
+    if (m->group_end() || in_directive) append_to_buffer(ctx->linefeed);
+
+    indentation -= m->tabs();
   }
 
   void Output_Nested::operator()(At_Rule* a)
@@ -300,7 +361,12 @@ namespace Sass {
       return;
     }
 
-    append_to_buffer(" {\n");
+    append_to_buffer(" {" + ctx->linefeed);
+
+    bool old_in_directive = in_directive;
+    in_directive = true;
+    in_keyframes = kwd.compare("@keyframes") == 0;
+
     ++indentation;
     decls = true;
     for (size_t i = 0, L = b->length(); i < L; ++i) {
@@ -308,7 +374,7 @@ namespace Sass {
       if (!stm->is_hoistable()) {
         if (!stm->block()) indent();
         stm->perform(this);
-        append_to_buffer("\n");
+        append_to_buffer(ctx->linefeed);
       }
     }
     --indentation;
@@ -318,28 +384,34 @@ namespace Sass {
       Statement* stm = (*b)[i];
       if (stm->is_hoistable()) {
         stm->perform(this);
-        append_to_buffer("\n");
+        // append_to_buffer(ctx->linefeed);
       }
     }
     if (decls) --indentation;
 
-    buffer.erase(buffer.length()-1);
-    if (ctx) ctx->source_map.remove_line();
-    if (b->has_hoistable()) {
+    while (buffer.substr(buffer.length()-ctx->linefeed.length()) == ctx->linefeed) {
       buffer.erase(buffer.length()-1);
       if (ctx) ctx->source_map.remove_line();
     }
-    append_to_buffer(" }\n");
+
+    in_directive = old_in_directive;
+    in_keyframes = false;
+
+    append_to_buffer(" }");
+
+    // Match Sass 3.4.9 behaviour
+    if (kwd.compare("@font-face") != 0 && kwd.compare("@keyframes") != 0) append_to_buffer(ctx->linefeed);
   }
 
   void Output_Nested::indent()
-  { append_to_buffer(string(2*indentation, ' ')); }
-
-  void Output_Nested::append_to_buffer(const string& text)
   {
-    buffer += text;
-    if (ctx && !ctx->_skip_source_map_update)
-      ctx->source_map.update_column(text);
+    string indent = "";
+    for (size_t i = 0; i < indentation; i++)
+      indent += ctx->indent;
+    append_to_buffer(indent);
   }
 
+  // compile output implementation
+  template class Output<Output_Nested>;
+
 }
