@@ -1,8 +1,10 @@
 #include <cctype>
 #include <cstddef>
 #include <iostream>
+#include <iomanip>
 #include "constants.hpp"
 #include "prelexer.hpp"
+#include "util.hpp"
 
 
 namespace Sass {
@@ -24,10 +26,10 @@ namespace Sass {
 
     // Match a single character satisfying the ctype predicates.
     const char* space(const char* src) { return std::isspace(*src) ? src+1 : 0; }
-    const char* alpha(const char* src) { return std::isalpha(*src) || !isascii(*src) ? src+1 : 0; }
+    const char* alpha(const char* src) { return std::isalpha(*src) || !Sass::Util::isAscii(*src) ? src+1 : 0; }
     const char* digit(const char* src) { return std::isdigit(*src) ? src+1 : 0; }
     const char* xdigit(const char* src) { return std::isxdigit(*src) ? src+1 : 0; }
-    const char* alnum(const char* src) { return std::isalnum(*src) || !isascii(*src) ? src+1 : 0; }
+    const char* alnum(const char* src) { return std::isalnum(*src) || !Sass::Util::isAscii(*src) ? src+1 : 0; }
     const char* punct(const char* src) { return std::ispunct(*src) ? src+1 : 0; }
     // Match multiple ctype characters.
     const char* spaces(const char* src) { return one_plus<space>(src); }
@@ -155,6 +157,13 @@ namespace Sass {
                                                 backslash_something > > >(src);
     }
 
+    const char* identifier_fragment(const char* src) {
+      return one_plus< alternatives< alnum,
+                                     exactly<'-'>,
+                                     exactly<'_'>,
+                                     backslash_something > >(src);
+    }
+
     // Match CSS selectors.
     const char* sel_ident(const char* src) {
       return sequence< optional< alternatives< exactly<'-'>, exactly<'|'> > >,
@@ -174,15 +183,16 @@ namespace Sass {
     // Match interpolant schemas
     const char* identifier_schema(const char* src) {
       // follows this pattern: (x*ix*)+ ... well, not quite
-      return one_plus< sequence< zero_plus< alternatives< identifier, exactly<'-'> > >,
+      return sequence< one_plus< sequence< zero_plus< alternatives< identifier, exactly<'-'> > >,
                                  interpolant,
-                                 zero_plus< alternatives< identifier, number, exactly<'-'> > > > >(src);
+                                 zero_plus< alternatives< identifier, number, exactly<'-'> > > > >,
+                       negate< exactly<'%'> > >(src);
     }
     const char* value_schema(const char* src) {
       // follows this pattern: ([xyz]*i[xyz]*)+
       return one_plus< sequence< zero_plus< alternatives< identifier, percentage, dimension, hex, number, string_constant > >,
                                  interpolant,
-                                 zero_plus< alternatives< identifier, percentage, dimension, hex, number, string_constant > > > >(src);
+                                 zero_plus< alternatives< identifier, percentage, dimension, hex, number, string_constant, exactly<'%'> > > > >(src);
     }
     const char* filename_schema(const char* src) {
       return one_plus< sequence< zero_plus< alternatives< identifier, number, exactly<'.'>, exactly<'/'> > >,
@@ -203,6 +213,22 @@ namespace Sass {
       return exactly<import_kwd>(src);
     }
 
+    const char* at_root(const char* src) {
+      return exactly<at_root_kwd>(src);
+    }
+
+    const char* with_directive(const char* src) {
+      return exactly<with_kwd>(src);
+    }
+
+    const char* without_directive(const char* src) {
+      return exactly<without_kwd>(src);
+    }
+
+    const char* until_closing_paren(const char* src) {
+      return until<')'>(src);
+    }
+
     const char* media(const char* src) {
       return exactly<media_kwd>(src);
     }
@@ -300,6 +326,14 @@ namespace Sass {
       return exactly<warn_kwd>(src);
     }
 
+    const char* err(const char* src) {
+      return exactly<error_kwd>(src);
+    }
+
+    const char* dbg(const char* src) {
+      return exactly<debug_kwd>(src);
+    }
+
     const char* directive(const char* src) {
       return sequence< exactly<'@'>, identifier >(src);
     }
@@ -319,6 +353,9 @@ namespace Sass {
     const char* hyphens_and_identifier(const char* src) {
       return sequence< zero_plus< exactly< '-' > >, identifier >(src);
     }
+    const char* hyphens_and_name(const char* src) {
+      return sequence< zero_plus< exactly< '-' > >, name >(src);
+    }
     const char* universal(const char* src) {
       return sequence< optional<namespace_prefix>, exactly<'*'> >(src);
     }
@@ -379,6 +416,11 @@ namespace Sass {
       ptrdiff_t len = p - src;
       return (len != 4 && len != 7) ? 0 : p;
     }
+    const char* hexa(const char* src) {
+      const char* p = sequence< exactly<'#'>, one_plus<xdigit> >(src);
+      ptrdiff_t len = p - src;
+      return (len != 4 && len != 7 && len != 9) ? 0 : p;
+    }
 
     const char* rgb_prefix(const char* src) {
       return exactly<rgb_kwd>(src);
@@ -514,14 +556,47 @@ namespace Sass {
 
     // match specific IE syntax
     const char* ie_progid(const char* src) {
-      return sequence < exactly<progid_kwd>, exactly<':'>, alternatives< identifier_schema, identifier >, one_plus< sequence< exactly<'.'>, alternatives< identifier_schema, identifier > > > >(src);
+      return sequence <
+        exactly<progid_kwd>,
+        exactly<':'>,
+        alternatives< identifier_schema, identifier >,
+        zero_plus< sequence<
+          exactly<'.'>,
+          alternatives< identifier_schema, identifier >
+        > >,
+        zero_plus < sequence<
+          exactly<'('>,
+          spaces_and_comments,
+          optional < sequence<
+            alternatives< variable, identifier_schema, identifier >,
+            spaces_and_comments,
+            exactly<'='>,
+            spaces_and_comments,
+            alternatives< variable, identifier_schema, identifier, string_constant, number, hexa >,
+            zero_plus< sequence<
+              spaces_and_comments,
+              exactly<','>,
+              spaces_and_comments,
+              sequence<
+                alternatives< variable, identifier_schema, identifier >,
+                spaces_and_comments,
+                exactly<'='>,
+                spaces_and_comments,
+                alternatives< variable, identifier_schema, identifier, string_constant, number, hexa >
+              >
+            > >
+          > >,
+          spaces_and_comments,
+          exactly<')'>,
+          spaces_and_comments
+        > >
+      >(src);
     }
     const char* ie_expression(const char* src) {
-      return exactly<expression_kwd>(src);
+      return sequence < exactly<expression_kwd>, delimited_by< '(', ')', true> >(src);
     }
-    // match any IE syntax
-    const char* ie_stuff(const char* src) {
-      return sequence< alternatives < ie_expression, ie_progid >, delimited_by<'(', ';', true> >(src);
+    const char* ie_property(const char* src) {
+      return alternatives < ie_expression, ie_progid >(src);
     }
 
     // const char* ie_args(const char* src) {
@@ -530,7 +605,7 @@ namespace Sass {
     // }
 
     const char* ie_keyword_arg(const char* src) {
-      return sequence< alternatives< variable, identifier_schema, identifier >, spaces_and_comments, exactly<'='>, spaces_and_comments, alternatives< variable, identifier_schema, identifier, number, hex > >(src);
+      return sequence< alternatives< variable, identifier_schema, identifier >, spaces_and_comments, exactly<'='>, spaces_and_comments, alternatives< variable, identifier_schema, identifier, string_constant, number, hexa > >(src);
     }
 
     // Path matching functions.
@@ -592,25 +667,38 @@ namespace Sass {
     }
 
     const char* url(const char* src) {
-      // using (more or less) the algorithm described at this url:
-      // http://www.w3.org/TR/css3-syntax/#consume-a-url-token
+      return chunk(src);
+    }
+
+    const char* static_string(const char* src) {
       const char* pos = src;
-      pos = zero_plus<spaces>(pos);
-      if (*pos == '"' || *pos == '\'') return string_constant(pos); // let the parser handle the rparen
-      while (*pos != ')') {
-        if (space(pos)) {
-          ++pos;
-          continue;
-        }
-        if (*pos == '\\') {
-          pos = ESCAPE(pos);
-          if (!pos) return 0; // invalid escape sequence
-          continue;
-        }
-        if (*pos == '"' || *pos == '\'' || *pos == '(') return 0;
-        ++pos;
-      }
-      return pos;
+      const char * s = string_constant(pos);
+      Token t(pos, s, Position(0, 0));
+      const unsigned int p = count_interval< interpolant >(t.begin, t.end);
+      return (p == 0) ? t.end : 0;
+    }
+
+    const char* static_component(const char* src) {
+      return alternatives< identifier,
+                           static_string,
+                           hex,
+                           number,
+                           sequence< exactly<'!'>, exactly<important_kwd> >
+                          >(src);
+    }
+
+    const char* static_value(const char* src) {
+      return sequence< static_component,
+                       zero_plus < sequence<
+                                   alternatives<
+                                     sequence< optional_spaces, exactly<'/'>, optional_spaces >,
+                                     sequence< optional_spaces, exactly<','>, optional_spaces >,
+                                     spaces
+                                   >,
+                                   static_component
+                       > >,
+                       alternatives< exactly<';'>, exactly<'}'> >
+                      >(src);
     }
   }
 }
