@@ -1,4 +1,7 @@
-#define SASS_PRELEXER
+#ifndef SASS_PRELEXER_H
+#define SASS_PRELEXER_H
+
+#include <cstring>
 
 namespace Sass {
   namespace Prelexer {
@@ -50,6 +53,14 @@ namespace Sass {
       return p == src ? 0 : p;
     }
 
+    // Match a sequence of characters up to the next newline.
+    template <char end>
+    const char* until(const char* src) {
+      if (*src == '\n' && exactly<end>(src)) return 0;
+      while (*src && *src != '\n' && !exactly<end>(src)) ++src;
+      return ++src;
+    }
+
     // Match a sequence of characters up to the next newline.
     template <const char* prefix>
     const char* to_endl(const char* src) {
@@ -172,11 +183,11 @@ namespace Sass {
               prelexer mx5, prelexer mx6,
               prelexer mx7>
     const char* alternatives(const char* src) {
-      const char* rslt = src;
-      (rslt = mx1(rslt)) || (rslt = mx2(rslt)) ||
-      (rslt = mx3(rslt)) || (rslt = mx4(rslt)) ||
-      (rslt = mx5(rslt)) || (rslt = mx6(rslt)) ||
-      (rslt = mx7(rslt));
+      const char* rslt;
+      (rslt = mx1(src)) || (rslt = mx2(src)) ||
+      (rslt = mx3(src)) || (rslt = mx4(src)) ||
+      (rslt = mx5(src)) || (rslt = mx6(src)) ||
+      (rslt = mx7(src));
       return rslt;
     }
 
@@ -186,11 +197,11 @@ namespace Sass {
               prelexer mx5, prelexer mx6,
               prelexer mx7, prelexer mx8>
     const char* alternatives(const char* src) {
-      const char* rslt = src;
-      (rslt = mx1(rslt)) || (rslt = mx2(rslt)) ||
-      (rslt = mx3(rslt)) || (rslt = mx4(rslt)) ||
-      (rslt = mx5(rslt)) || (rslt = mx6(rslt)) ||
-      (rslt = mx7(rslt)) || (rslt = mx8(rslt));
+      const char* rslt;
+      (rslt = mx1(src)) || (rslt = mx2(src)) ||
+      (rslt = mx3(src)) || (rslt = mx4(src)) ||
+      (rslt = mx5(src)) || (rslt = mx6(src)) ||
+      (rslt = mx7(src)) || (rslt = mx8(src));
       return rslt;
     }
 
@@ -324,8 +335,10 @@ namespace Sass {
     const char* custom_property_name(const char* src);
     // Match a CSS identifier.
     const char* identifier(const char* src);
+    const char* identifier_fragment(const char* src);
     // Match selector names.
     const char* sel_ident(const char* src);
+    const char* until_closing_paren(const char* src);
     // Match interpolant schemas
     const char* identifier_schema(const char* src);
     const char* value_schema(const char* src);
@@ -337,6 +350,9 @@ namespace Sass {
     // Match CSS '@' keywords.
     const char* at_keyword(const char* src);
     const char* import(const char* src);
+    const char* at_root(const char* src);
+    const char* with_directive(const char* src);
+    const char* without_directive(const char* src);
     const char* media(const char* src);
     const char* supports(const char* src);
     const char* keyframes(const char* src);
@@ -363,6 +379,8 @@ namespace Sass {
     const char* while_directive(const char* src);
 
     const char* warn(const char* src);
+    const char* err(const char* src);
+    const char* dbg(const char* src);
 
     const char* directive(const char* src);
     const char* at_keyword(const char* src);
@@ -373,6 +391,7 @@ namespace Sass {
     const char* namespace_prefix(const char* src);
     const char* type_selector(const char* src);
     const char* hyphens_and_identifier(const char* src);
+    const char* hyphens_and_name(const char* src);
     const char* universal(const char* src);
     // Match CSS id names.
     const char* id_name(const char* src);
@@ -391,6 +410,7 @@ namespace Sass {
     const char* percentage(const char* src);
     const char* dimension(const char* src);
     const char* hex(const char* src);
+    const char* hexa(const char* src);
     const char* rgb_prefix(const char* src);
     // Match CSS uri specifiers.
     const char* uri_prefix(const char* src);
@@ -442,8 +462,9 @@ namespace Sass {
     const char* lte_op(const char* src);
 
     // IE stuff
-    const char* ie_stuff(const char* src);
-    const char* ie_args(const char* src);
+    const char* ie_progid(const char* src);
+    const char* ie_expression(const char* src);
+    const char* ie_property(const char* src);
     const char* ie_keyword_arg(const char* src);
 
     // match urls
@@ -453,6 +474,11 @@ namespace Sass {
     const char* folder(const char* src);
     const char* folders(const char* src);
 
+
+    const char* static_string(const char* src);
+    const char* static_component(const char* src);
+    const char* static_value(const char* src);
+
     // Utility functions for finding and counting characters in a string.
     template<char c>
     const char* find_first(const char* src) {
@@ -500,3 +526,5 @@ namespace Sass {
     const char* chunk(const char* src);
   }
 }
+
+#endif
