@@ -11,9 +11,8 @@ class SassSpec::Runner
   end
 
   def run
-    unless @options[:silent]
+    unless @options[:silent] || @options[:tap]
       puts "Recursively searching under directory '#{@options[:spec_directory]}' for test files to test '#{@options[:sass_executable]}' with."
-
       stdout, stderr, status = Open3.capture3("#{@options[:sass_executable]} -v")
       puts stdout
     end
@@ -26,6 +25,11 @@ class SassSpec::Runner
       minioptions.push '--verbose'
     end
 
+    if @options[:tap]
+      require 'minitap'
+      Minitest.reporter = Minitap::TapY
+    end
+
     exit Minitest.run(minioptions)
   end
 
@@ -35,7 +39,9 @@ class SassSpec::Runner
     Dir.glob(glob) do |filename|
       expected = Pathname.new(filename).dirname.join(@options[:expected_file])
       input = Pathname.new(filename)
-      cases.push SassSpec::TestCase.new(input.realpath(), expected.realpath(), @options)
+      if filename.include?(@options[:filter])
+        cases.push SassSpec::TestCase.new(input.realpath(), expected.realpath(), @options)
+      end
     end
     cases
   end
