@@ -28,8 +28,8 @@ class SassSpec::TestCase
       return @output
     end
     stdout, stderr, status = Open3.capture3("#{@options[:sass_executable]} #{@input_path}")
-    stdout = _clean_output(stdout)
-    @output ||= [stdout, stderr, status]
+    cleaned = _clean_output(stdout)
+    @output ||= [stdout, cleaned, stderr, status]
   end
 
   def expected
